<?php
/*
DynPage V1.35 - A simple Content Management System
Copyright (C) 2009-2019 Matthias Wiede <mwiede@mwiede.de>

This program is free software; you can redistribute it and/or modify 
it under the terms of the GNU General Public License as published by 
the Free Software Foundation; either version 3 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranty of  MERCHANTABILITY 
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
for more details.

You should have received a copy of the GNU General Public License 
along with this program; if not, see http://www.gnu.org/licenses.
*/

error_reporting(0);
$dynPageVersion = "1.35";

// We dont need mysql support
$fNoDB = true; 
require_once ("main.inc.php");

function isDemo () {
	return ((int) getConf ("is_demo")!=0);
}

function isLogin (&$userData=null) {
  global $userList;
	if (isDemo ()) {
		$_SESSION["dynpage_is_login"] = 1;
		$_SESSION["dynpage_is_demo"] = 1;
	}
	if (isset ($_SESSION["dynpage_is_login"])) {
    if (is_array ($_SESSION["dynpage_is_login"])) {
      $userData = $_SESSION["dynpage_is_login"];
      if (!isset ($userList[$userData["email"]])) {
        // If user is not in user list exit
        unset ($_SESSION["dynpage_is_login"]);
        die ("Your login was removed from DynPage.");
      }
      return 2; // User
    }
    return  1;  // Super admin
  }
  return false; // No login
}

function getPageData ($filename, $fContent=true) {	
	$pageData = array ();
	$pageData["filename"]=$filename;
	$pageData["filetime"]=filemtime ($filename);
	$pageData["name"]=basename ($filename);
	$pageData["url"]="";
	$fh = fopen($filename, 'r');
	if ($fh) {
		if (!feof($fh)) {
			$line = fgets($fh);
			if (preg_match ("/<!--(.*?)-->/i", $line, $comment)==1) {
				$tokenList = explode (";", $comment[1]);
				foreach ($tokenList as $token) {
					list($key, $value) = explode ("=", $token);					
					$pageData[trim ($key)]=$value;
				}

				// Checks
				if (trim ($pageData["name"])=="")
					$pageData["name"]=basename ($filename);					
				if (trim ($pageData["filetime"])=="")
					$pageData["filetime"]=filemtime ($filename);
        
				// Read content
        if ($fContent) {
  				$bytesRest = filesize ($filename)-ftell ($fh);
  				if ($bytesRest>0)
  					$pageData["content"] = fread ($fh, $bytesRest);
  				else
  					$pageData["content"] = "";
        }
			}
			else
			{
				// No comment line
        if ($fContent) {
          $pageData["content"] = fread ($fh, filesize ($filename));
        }
			}
		}
		fclose ($fh);
	}
	return $pageData;
}

function pageSortFunc ($pageData1, $pageData2) {
	return strcmp ($pageData1["name"], $pageData2["name"]);
}

function loadPageList ($fRefresh=false, $fContent=true) {
	// Iterate all pages (session saved!)
	if (!isset ($_SESSION["pageFileList"]) || $fRefresh) {
		$i = 0;
		$pageFileList = array ();
		if ($dh = opendir(ROOT_PATH."/".getConf ("content_path"))) {
			while (false !== ($filename = readdir($dh))) {
				// $ext = preg_replace("^.+\\.([^.]+)$", "\\1", $filename);
        $ext = pathinfo($filename, PATHINFO_EXTENSION);
				if (strtolower ($ext)!=getConf ("file_ext", "htm"))
					continue;
				if (is_file (ROOT_PATH."/".getConf ("content_path")."/".$filename)) {			
					$i++;
					$pageFileList[$i]=getPageData (ROOT_PATH."/".getConf ("content_path")."/".$filename, $fContent);				
				}
			}
			closedir($dh);
		}
		uasort ($pageFileList, "pageSortFunc");
		$_SESSION["pageFileList"]=$pageFileList;
	}
}

$userConfFilename = ROOT_PATH."/".getConf ("conf_path")."/".getConf ("userconf_file");

function updateUserDatabase () {
  global $userList, $userConfFilename;
  $fh = fopen ($userConfFilename, "w+");                
  if ($fh) {
    fwrite ($fh, "<?php\r\n"); 
    foreach ($userList as $email2 => $userData) {
      $line = "// ".strtolower ($userData["email"]).";".$userData["hash"].";".$userData["create_pages"].";".$userData["file_uploads"].";".$userData["delete_pages"].";".$userData["all_pages"].";;;;;";
      foreach ($userData["fileList"] as $idx => $filename) {
        $line.=$filename.";";
      }
    
      fwrite ($fh, $line."\r\n");
    }
    fwrite ($fh, chr(63).">"); 
    fclose ($fh);
    return true;
  }
  return false;
}

// Load all pages
loadPageList (!getConf ("cache_page_list"));


// Looad existing users
$userList = array ();
if (is_file ($userConfFilename)) {
  $fh = fopen ($userConfFilename, "r");  
  if ($fh) {
  	$idx = 0;
    while (!feof($fh)) {        
      $line = trim (fgets($fh), "\r\n /");
      
      $list = explode (";", $line);
      if (count ($list)<7)
        continue;
      
      $userData = array (
        "email" => $list[0],
        "hash" => $list[1],
        "create_pages" => $list[2],
        "file_uploads" => $list[3],
        "delete_pages" => $list[4],
        "all_pages" => $list[5],
        "line" => $line,
      );
      $fileList = array ();
      $fileList2 = array ();
      for ($i=10;$i<count ($list);$i++) {
        $fileList[$i-10]=$list[$i];
        $fileList2[$list[$i]]=$list[$i];
      }
      $userData["fileList"]=$fileList;
      $userData["fileList2"]=$fileList2;
      $userList[$userData["email"]]=$userData;
      $idx++;      
  	}
    fclose ($fh);
  }
}

// Get menu page
$page = getParamFromSession ("page");
if ($page=="")
	$page = "page_new";

// Get page id from filename
$pidFromFile = 0;
if (isParam ("file")) {
	foreach ($_SESSION["pageFileList"] as $i => $pageData) {
		if (basename ($pageData["filename"])==getParam ("file")) {
			$pidFromFile = $i;
			$page = "page_edit_".$i;
		}
	}
	if ($pidFromFile==0)
		$page = "page_edit_unknown";
}
$pagePath = explode ("_", $page);

// Get content file
$contentFilename = "dynpage.".$page.".inc.php";
if (!file_exists ($contentFilename)) {
	$contentFilename = "";
	if (count ($pagePath)>1 && (int)$pagePath[count($pagePath)-1]!=0) {
		for ($i=0;$i<count($pagePath)-1;$i++) {
			if ($contentFilename!="")
				$contentFilename.="_";
			$contentFilename.=$pagePath[$i];
		}
		$contentFilename = "dynpage.".$contentFilename.".inc.php";
	}
	if (!file_exists ($contentFilename))
		$contentFilename = "dynpage.empty.inc.php";
}

// Get logic file
$onLoad = "";
$logicFilename = "logic/dynpage.".$page.".inc.php";
if (!file_exists ($logicFilename)) {
	$logicFilename = "";
	$cnt = count ($pagePath);
	while ($cnt>0) {
		$logicFilename = "";
		for ($i=0;$i<$cnt;$i++) {
			if ($logicFilename!="")
				$logicFilename.="_";
			$logicFilename.=$pagePath[$i];
		}
		$logicFilename = "logic/dynpage.".$logicFilename.".inc.php";
		if (file_exists ($logicFilename))
			break;
		$cnt--;
	}
}

// Check login
$loginUserData = array ();
if ($page=="request_pwd") {
	// No login required ...
}
else
if ($ret = isLogin ($loginUserData)) {
	define ('DYNPAGE_LOGIN', $ret);
}
else {	
	if ((!isset ($_SESSION["dynpage_login_url"]) || $_SESSION["dynpage_login_url"]=="") && 
		$page!="login" && $page!="logout") {
		$_SESSION["dynpage_login_url"] = $_SERVER['REQUEST_URI'];
	}	
	$logicFilename   = "logic/dynpage.login.inc.php";
	$contentFilename = "dynpage.login.inc.php";
}

?>
