<?php
/*
DynPage V1.35 - A simple Content Management System
Copyright (C) 2009-2019 Matthias Wiede <mwiede@mwiede.de>

This program is free software; you can redistribute it and/or modify 
it under the terms of the GNU General Public License as published by 
the Free Software Foundation; either version 3 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranty of  MERCHANTABILITY 
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
for more details.

You should have received a copy of the GNU General Public License 
along with this program; if not, see http://www.gnu.org/licenses.
*/

function stdround($num, $d=0) {
    return round($num + 0.0001 / pow(10, $d), $d);
}

function formatCurrency ($price, $precision=2, $fSepThousand=false, $fWithSymbol=false) {
	$str = sprintf(getConf ("currency_format", "%01.".$precision."f"),stdround ($price,$precision) );                
	$str2 = "";
	if ($fSepThousand)
		$str2 = number_format($price, $precision, ',', '.');
	else
		$str2 = str_replace(".", ",", $str);
	if ($fWithSymbol)
		$str2.=" ".getConf ("currency_symbol");
	return $str2;
}

function formatCurrencyDeluxe ($price) {
	$str = formatCurrency ($price, 2, true, true);
	if ($price<0)
		$str = "<font color=\"red\">".$str."</font>";
	return $str;
}

function parseCurrency ($price) {
	$value2 = 0;
	sscanf(str_replace(",", ".", $price), "%f", $value2);
	return str_replace(",", ".", stdround ($value2, 2));
}

function formatNumber ($value, $precision=2) {
	$value2 = 0;
	sscanf(str_replace(",", ".", $value), "%f", $value2);
	return str_replace(".", ",", stdround ($value2, $precision));
}

function getDateStr ($distDays=0, $timestamp=0) {
	if ($timestamp==0)
		$timestamp = time();
	return date ("Y-d-m", $timestamp + $distDays*3600*24);
}

function getDateTimeStr ($distDays=0, $timestamp=0) {
	if ($timestamp==0)
		$timestamp = time();	
	return date ("Y-m-d Y H:i:s", time () + $distDays*3600*24);
}

function checkDateStr ($dateStr) {
	$parts = explode("-", $dateStr); 
	if ((count ($parts)!=3) || !checkdate ($parts[1], $parts[2], $parts[0]))
		return false;
	return true;		
}

function convertMySQLDate2TimeValue ($mySQLDate) {
	if ($mySQLDate!="0000-00-00")
	{
		$str = explode("-",$mySQLDate);
		if (count ($str)==3)
			return mktime(0, 0, 0, $str[1], $str[2], $str[0]);
	}
	return 0;
}

function formatMySQLDate ($mySQLDate) { 
	if ($mySQLDate=="0000-00-00")
		$mySQLDate= "";
    $str = explode("-",$mySQLDate); 
	if (count ($str)==3)
		return sprintf ("%04d-%02d-%02d", $str[0], $str[1], $str[2]);
	return $mySQLDate;
} 

function parseMySQALDate ($dateStr) { 
    $str = explode("-",$dateStr); 
	if (count ($str)==3) {
		if ($str[0]<100)
			$str[0]+=2000;
	    return $str[0].'-'.$str[1].'-'.$str[2]; 
	}
	return $dateStr;
} 

function formatMySQLTimestamp ($mySQLTimestamp, $fOnlyDate=false, $fBlankZero=false) {
	list($year, $month, $day, $hour, $minute, $second) =  sscanf($mySQLTimestamp, "%04d-%02d-%02d %02d:%02d:%02d");
	if (!$fOnlyDate)
		$str = sprintf ("%04d-%02d-%02d %02d:%02d:%02d", $year, $month, $day, $hour, $minute, $second);
	else
		$str = sprintf ("%04d-%02d-%02d", $year, $month, $day);
	if ($fBlankZero) {
		if ($str=="0000-00-00")
			$str="";
		else
		if ($str=="0000-00-00 00:00:00")
			$str="";
	}
	return $str;
} 

function parseMySQLTimestamp ($dateTimeStr) {
	list($year, $month, $day, $hour,$minute,$second) =  sscanf($dateTimeStr, "%04d-%02d-%02d %02d:%02d:%02d");
	return sprintf ("%04d-%02d-%02d %02d:%02d:%02d", $year, $month, $day, $hour, $minute, $second);
}

function compareMySQLDate ($dateStart, $dateStop) {
	$time = convertMySQLDate2TimeValue ($dateStart);
	$time2 = convertMySQLDate2TimeValue ($dateStop);
	if ($time2<$time && $time2!=0)
		return false;
	return true;
}
?>
