<?php
/*
DynPage V1.35 - A simple Content Management System
Copyright (C) 2009-2019 Matthias Wiede <mwiede@mwiede.de>

This program is free software; you can redistribute it and/or modify 
it under the terms of the GNU General Public License as published by 
the Free Software Foundation; either version 3 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranty of  MERCHANTABILITY 
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
for more details.

You should have received a copy of the GNU General Public License 
along with this program; if not, see http://www.gnu.org/licenses.
*/

function isURL($url) {
	if (!preg_match('#^http\\:\\/\\/[a-z0-9\-]+\.([a-z0-9\-]+\.)?[a-z]+#i', $url))
        return false;
	return true;
} 

function addPTag (&$str) {
	$checkHT = array ("<table","<div","<h","<p","{");
	$found = false;
	for ($i=0;$i<count($checkHT) && !$found;$i++)
		$found = !strncmp ($str, $checkHT[$i], strlen ($checkHT[$i]));	
	if ($str!="" && !$found) {
		$str2 = "<p>".$str."</p>";
		$str = $str2;
		return true;	
	}
	return false;
}

function loadInputParams (&$paramDefHT, &$keyValueHT, $pos, &$errorHT, $fNew, $fShowErrors=true) {
	$errCnt=0;
	reset ($paramDefHT);
	foreach ($paramDefHT as $key => $flags) {
		$fMandatory   = false;
		$fInput		    = true;
		$fUseDefault  = true;
		$defaultValue = "";
		$tokenHT      = explode (";",$flags);
		$fieldType		= "s";
		$fieldPos	    = 0;
		$key2         = $key;

		foreach ($tokenHT as $i => $value) {
      
			switch ($i) {
				case 0:
					$fieldPos = $value;			
					break;

				case 1: 								
					$fieldType=$value;
					if ($fieldType=="n" || $fieldType=="f" || $fieldType=="c")
						$defaultValue=0;
					else
					if ($fieldType=="d" || $fieldType=="date")
						$defaultValue="0000-00-00";
					else
					if ($fieldType=="t" || $fieldType=="time")
						$defaultValue="0000-00-00 00:00:00";
					break;

				case 2:					
					$fMandatory = ($value=="m");
					if ($value == "$" || (!$fNew && !isParam ($key)))					
						$fInput = false;
					break;
						
				case 3:
					if (!empty ($value))
						$defaultValue = $value;
					break;
			}
		}
		if ($fieldPos!=$pos)
			continue;

		if ($fInput) {
			$errorHT[$key2]="";
			
			if (isParam ($key))
				$keyValueHT[$key2]=getParam($key); 
			else {
				$keyValueHT[$key2]=$defaultValue;
				continue;
			}
      
      $keyValueHT[$key2] = trim ($keyValueHT[$key2]);

			// Filter HTML trash
			if ($keyValueHT[$key2]=="<p>&nbsp;</p>")
				$keyValueHT[$key2]="";

			switch ($fieldType) {
				case "n":
					$keyValueHT[$key2] =(int) $keyValueHT[$key2];
					break;
				case "f":
					if ($keyValueHT[$key2]=="1" || ($keyValueHT[$key2]!="" && $keyValueHT[$key2]!="0"))
						$keyValueHT[$key2]=1;
					else
						$keyValueHT[$key2]=0;
					break;
				
				// HTML
				case "html":
					addPTag ($keyValueHT[$key2]);
					break;
					
				// Currency
				case "c":
					$keyValueHT[$key2] = parseCurrency ($keyValueHT[$key2]);
					break;
					
				// eMail
				case "e":
					if (($keyValueHT[$key2]!="") && 
						!preg_match("/^([a-zA-Z0-9\.\_\-]+)@([a-zA-Z0-9\.\-]+\.[A-Za-z][A-Za-z]+)$/", $keyValueHT[$key2])) {
						if ($fShowErrors) {								
							$errorHT[$key2]="&nbsp;<span class=\"error\">invalid E-Mail</span>";
							$errCnt++;
						}
					}
					else
						$keyValueHT[$key2]=strtolower ($keyValueHT[$key2]);
					break;

				// Clock
				case "clock":
					if (($keyValueHT[$key2]!="") && 
						!preg_match("/^([01][0-9]|2[0-3]):[0-5][0-9]/", $keyValueHT[$key2])) {
						if ($fShowErrors) {								
							$errorHT[$key2]="<img style=\"vertical-align:middle;\" src=\"images/arrow_err.gif\"></images>";
							$errCnt++;
						}
					}
					break;

				// Date
				case "d":;
				case "date":
					if ($keyValueHT[$key2]!="") {
						if ((!preg_match("/([0-9]{1,2}).([0-9]{1,2}).([0-9]{2,4})/",$keyValueHT[$key2]) || !checkDateStr ($keyValueHT[$key2])) && $fShowErrors) {								
							$errorHT[$key2]="<img style=\"vertical-align:middle;\" src=\"images/arrow_err.gif\"></images>";
							$errCnt++;
						}
						else
							$keyValueHT[$key2]=parseMySQLDate ($keyValueHT[$key2]);
					}
					else
						$keyValueHT[$key2]=$defaultValue;
					break;

				// ZIP (PLZ)
				case "z":
					if (($keyValueHT[$key2]!="") && 
					(((int) $keyValueHT[$key2] <= 1000) || ((int) $keyValueHT[$key2]) > 99998)) {								
						if ($fShowErrors) {
							$errorHT[$key2]="&nbsp;<span class=\"error\">invalid ZIP</span>";
							$errCnt++;
						}
					}				
					break;

				// Filename
				case "fn":
					if (($keyValueHT[$key2]!="") && 
						!preg_match("/^[a-zA-Z0-9._-]*$/", $keyValueHT[$key2])) {
						if ($fShowErrors) {								
							$errorHT[$key2]="&nbsp;<span class=\"error\">Invalid filename</span>";
							$errCnt++;
						}
					}
					else
						$keyValueHT[$key2]=$keyValueHT[$key2];						
					break;
			};
			if (empty ($keyValueHT[$key2]) && $fMandatory) {							
				if ($fShowErrors) {
					$errorHT[$key2]="<img style=\"vertical-align:middle;\" src=\"images/arrow_err.gif\">";
					$errCnt++;
				}
			}
		}
		else
		if ($fNew) {				
			$keyValueHT[$key2]=$defaultValue;
		}
	}
	return $errCnt;
}

function printDropDownSelection ($table, $idColumn, $selectedId, $columnList=array ("name"), $whereClause="", $inputName="", $onLoad="", $fAllowNoSeletion=true, $noSelectionText="-- SELECT --", $width="auto") {
	global $dbLink;
	$html = "";
	$sql = "select ".$idColumn;
	foreach ($columnList as $column)
		$sql.=",".$column;
	$sql.=" from ".$table;
	if ($whereClause!="")
		$sql.=" ".$whereClause; 
	$dbResult = mySQLQuery ($sql, $dbLink);
	if ($dbResult) {
		while ($dbRow=mysql_fetch_array ($dbResult))
		{
			$extra = "";
			if ($selectedId==$dbRow[$idColumn])
				$extra = "selected";
			$text = "";				
			foreach ($columnList as $column) {
				if ($text!="")
					$text.=" ";
				$text.=$dbRow[$column];
			}
			$html.="<option ".$extra." value=\"".$dbRow[$idColumn]."\">".$text."</option>\n";
		}
		mysql_free_result ($dbResult);
	}
	print ("<select style=\"width:".$width.";\" ".$onLoad." name=\"".($inputName=="" ? $idColumn : $inputName)."\">\n");
	if ($fAllowNoSeletion)
		print ("<option value=\"\">".htmlspecialchars ($noSelectionText)."</option>\n");
	print ($html);
	print ("</select>\n");	
}

function printDropDownSelectionLocked ($table, $idColumn, $selectedId, $columnList=array ("name"), $whereClause="", $inputName="", $onLoad="", $fAllowNoSeletion=true, $noSelectionText="-- SELECT --", $width="auto") {
	global $dbLink;
	$text = "";
	$sql  = "select ".$idColumn;
	foreach ($columnList as $column)
		$sql.=",".$column;
	$sql.=" from ".$table." where ".$idColumn."=".$selectedId;
	$dbResult = mySQLQuery ($sql, $dbLink);
	if ($dbResult) {
		while ($dbRow=mysql_fetch_array ($dbResult)) {
			$text = "";				
			foreach ($columnList as $column) {
				if ($text!="")
					$text.=" ";
				$text.=$dbRow[$column];
			}
			if ($selectedId==$dbRow[$idColumn])
				break;
		}
		mysql_free_result ($dbResult);
	}
	print (htmlspecialchars ($text));
}


function printYesNoDropDownSelection ($inputName, $fYes) {
	print ("<select name=\"".$inputName."\">\n");
	if (!$fYes) {
		print ("<option value=\"0\" selected>".getYesNoName(0)."</option>\n");
		print ("<option value=\"1\">".getYesNoName(1)."</option>\n");
	}
	else {
		print ("<option value=\"0\">".getYesNoName(0)."</option>\n");
		print ("<option value=\"1\" selected>".getYesNoName(1)."</option>\n");
	}
	print ("</select>\n");
}


function printYesNoDropDownSelectionLocked ($inputName, $fYes) {
	print (getYesNoName($fYes));
}

function printKeyValueDropDownSelection (&$keyValueHT, $inputName, $selectedKey, $showKey="", $onLoad="", $fAllowNoSeletion=false, $noSelectionText="-- SELECT --", $width="auto") {
	print ("<select style=\"width:".$width.";\" ".$onLoad." name=\"".$inputName."\">\n");
	if ($fAllowNoSeletion) {
		print ("<option value=\"\">".htmlspecialchars ($noSelectionText)."</option>\n");
	}
	foreach ($keyValueHT as $key => $value) {
		if ($showKey!="")
			$text = $value[$showKey];
		else
			$text = $value;			
		$extra = "";
		if ($key==$selectedKey)
			$extra = "selected";
		print ("<option ".$extra." value=\"".$key."\">".$text."</option>\n");
	}
	print ("</select>\n");
}

function printKeyValueDropDownSelectionLocked (&$keyValueHT, $inputName, $selectedKey, $showKey="", $onLoad="", $fAllowNoSeletion=false, $noSelectionText="-- SELECT --", $width="auto") {
	$text = "";
	foreach ($keyValueHT as $key => $value) {
		if ($showKey!="")
			$text = $value[$showKey];
		else
			$text = $value;			
		$extra = "";
		if ($key==$selectedKey)
			break;
	}
	print (htmlspecialchars ($text));
}

function printChecked ($flag, $fLocked=false) {
	if (!$fLocked) {
		if ($flag)
			print ("checked");
	}
	else {
		if ($flag)
			print ("(1)");
		else
			print ("(0)");
	}
}

?>
