<?php
/*
DynPage V1.35 - A simple Content Management System
Copyright (C) 2009-2019 Matthias Wiede <mwiede@mwiede.de>

This program is free software; you can redistribute it and/or modify 
it under the terms of the GNU General Public License as published by 
the Free Software Foundation; either version 3 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranty of  MERCHANTABILITY 
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
for more details.

You should have received a copy of the GNU General Public License 
along with this program; if not, see http://www.gnu.org/licenses.
*/

function encodeQuotedPrintable ($input, $line_max = 76) {
	$hex    = array('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'); 
	$lines  = preg_split("/(?:\r\n|\r|\n)/", $input); 
	$eol    = "\n"; 
	$escape = "="; 
	$output = ""; 
	  
	while( list(, $line) = each($lines) ) { 
		//$line  = rtrim($line); // remove trailing white space -> no =20\r\n necessary 
		$linlen  = strlen($line); 
		$newline = ""; 
		for ($i = 0; $i < $linlen; $i++) { 
			$c   = substr($line, $i, 1); 
			$dec = ord($c); 
			if ( ($dec == 32) && ($i == ($linlen - 1)) ) {  // convert space at eol only 
				$c = "=20"; 
			}
			elseif (($dec == 61) || ($dec < 32 ) || ($dec > 126)) {  // always encode "\t", which is *not* required 
				$h2 = floor($dec/16); $h1 = floor($dec%16); 
				$c = $escape.$hex["$h2"].$hex["$h1"]; 
			} 
			if ((strlen($newline) + strlen($c)) >= $line_max) { // CRLF is not counted 
				$output .= $newline.$escape.$eol; // soft line break; " =\r\n" is okay 
				$newline = ""; 
			} 
			$newline .= $c; 
		} // end of for 
		$output .= $newline.$eol; 
	} 
  
	return trim($output); 
} 


function sendMail ($from, $to, $subject, &$content_text, &$content_html, $reply_to="", $filename="") {
	$boundary_id   = time()."_".rand ();
	$boundary_id2  = time()."_".rand ();

	$keys = array (
		"[boundary_id]", 
		"[boundary_id2]",
		"[content_length_text]",
		"[content_length_html]",
		"[content_length_file]",
		"[content_text]",			
		"[content_html]",
		"[content_file]",
		"[filename]");

	$content_text_enc    = encodeQuotedPrintable ($content_text);
	$content_html_enc    = encodeQuotedPrintable ($content_html);
	$content_file_enc    = "";
	$content_length_text = strlen ($content_text_enc);
	$content_length_html = strlen ($content_html_enc);
	$content_length_file = 0;

	if ($filename!="" && file_exists ($filename)) {
		if ($handle = fopen($filename, "rb")) {
			$content_file = fread ($handle, filesize ($filename));
			$content_file_enc = chunk_split(base64_encode($content_file),76,"\n");
			$content_length_file = filesize ($filename);
			fclose ($handle);
		}
	}
	$values = array (
		$boundary_id, 
		$boundary_id2,
		$content_length_text,
		$content_length_html,
		$content_length_file,		
		$content_text_enc,			
		$content_html_enc,
		$content_file_enc,
		basename($filename));
	// windows-1252	

	$body = "------=_NextPart_[boundary_id]\n".
			"Content-Type: text/plain; charset=utf-8\n".
			"Content-Transfer-Encoding: quoted-printable\n".
			"Content-Length: [content_length_text]\n".
			"\n".
			"[content_text]\n".
			"------=_NextPart_[boundary_id]\n".
			"Content-Type: text/html; charset=utf-8\n".
			"Content-Transfer-Encoding: quoted-printable\n".
			"Content-Length: [content_length_html]\n".
			"\n".
			"[content_html]\n".
			"\n".
			"------=_NextPart_[boundary_id]--";

	$body2 = "This is a multi-part message in MIME format.\n".
			"\n".
			"------=_NextPart_[boundary_id]\n".
			"Content-Type: multipart/alternative;boundary=\"----=_NextPart_[boundary_id2]\"\n".
			"\n".
			"\n".
			"------=_NextPart_[boundary_id2]\n".
			"Content-Type: text/plain; charset=utf-8\n".
			"Content-Transfer-Encoding: quoted-printable\n".
			"Content-Length: [content_length_text]\n".
			"\n".
			"[content_text]\n".
			"------=_NextPart_[boundary_id2]\n".
			"Content-Type: text/html; charset=utf-8\n".
			"Content-Transfer-Encoding: quoted-printable\n".
			"Content-Length: [content_length_html]\n".
			"\n".
			"[content_html]\n".
			"\n".
			"------=_NextPart_[boundary_id2]--\n".
			"\n".
			"------=_NextPart_[boundary_id]\n".
			"Content-Type: application/octet-stream; name=[filename]\n".
			"Content-Transfer-Encoding: base64\n".
			"Content-Disposition: attachment; filename=[filename]\n".
			"Content-Length: [content_length_file]\n".
			"\n".
			"[content_file]\n".
			"\n".
			"------=_NextPart_[boundary_id]--";
	
	if ($content_length_file>0)
		$body = str_replace($keys, $values, $body2);
	else
		$body = str_replace($keys, $values, $body);

	// Send mail
	$extra = "";
	if ($reply_to!="")
		$extra="Bcc: ".$reply_to."\n";

	if ($content_length_file>0)
		$content_type = "multipart/mixed"; 
	else
		$content_type = "multipart/alternative"; // // Mailclient decide if TEXT or HTML
	return mail(
			$to, 
			$subject, 
			$body,
			$extra."From: ".$from."\nMIME-Version: 1.0\n".
			"Content-Type: ".$content_type.";boundary=\"----=_NextPart_".$boundary_id."\"");
}
?>
