<?php
/*
DynPage V1.35 - A simple Content Management System
Copyright (C) 2009-2019 Matthias Wiede <mwiede@mwiede.de>

This program is free software; you can redistribute it and/or modify 
it under the terms of the GNU General Public License as published by 
the Free Software Foundation; either version 3 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranty of  MERCHANTABILITY 
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
for more details.

You should have received a copy of the GNU General Public License 
along with this program; if not, see http://www.gnu.org/licenses.
*/
if (!defined('PHP_VERSION_ID')) {
  $version = explode('.', PHP_VERSION);
  define('PHP_VERSION_ID', ($version[0] * 10000 + $version[1] * 100 + $version[2]));
}
session_start ();
require_once ("config_global.inc.php");

if (PHP_VERSION_ID>=70200)
  $CONF_VAR["use_ckfinder"]=0; // Not compatible to PHP > 7.2.0
  
/* Set language environment */
// setlocale (LC_ALL, 'de_DE@euro', 'de_DE', 'de', 'ge');
date_default_timezone_set ($CONF_VAR["timezone"]);

function utf8_strtolower ($str) {
	return utf8_encode(strtolower (utf8_decode($str)));
}
function utf8_strtoupper ($str) {
	return utf8_encode(strtoupper (utf8_decode($str)));
}

/* Extract slashes "\\" */
function stripslashesDeep ($value) {
	if (function_exists("get_magic_quotes_gpc") && get_magic_quotes_gpc())
		$value = is_array($value) ? array_map('stripslashesDeep', $value) : stripslashes($value);
	return $value;
}

/* Check if parameter exists */
function isParam ($name) {
	global $_GET, $_POST;
	return isset ($_GET[$name]) || isset ($_POST[$name]);
}

/* Get parameter variable or empty */
function getParam ($name, $default="") {
	global $_GET, $_POST;
	if (isset ($_POST[$name]))
		return stripslashesDeep($_POST[$name]);
	if (isset ($_GET[$name]))		
		return stripslashesDeep($_GET[$name]);
	return $default;
}

/* Get value from array or default value if not set */
function getArrayValue (&$array, $key, $defaultValue="", $subKey="") {
	if (isset ($array[$key])) {
		if ($subKey!="") {	
			if (isset ($array[$key][$subKey]))
				return $array[$key][$subKey];
			else
				return $defaultValue;
		}
		return $array[$key];		
	}
	return $defaultValue;	
}

/* Replace array values */
function replaceArray (&$dstHT, &$srcHT) {
	while (list ($key, $value) = each ($srcHT)) {
		$dstHT[$key]=$value;
	}
}

/* Print error text */
function printError ($text="", $fPTag=true, $fHTML=false) {
	global $errText;
	if ($text=="" && !empty ($errText))
		$text=$errText;
	if ($text!="") {
		if (!$fHTML)
			$text = htmlspecialchars ($text);
		if ($fPTag)
			print ("<p class=\"error\">".$text."</p>");
		else
			print ("<span class=\"error\">".$text."</span>");
	}
}

/* Print log message in HTML format */
function printLog ($text, $fMarked=false) {
	if ($fMarked)
		print ("<span class=\"marked\">".htmlspecialchars ($text)."</span><br>\n");
	else
		print (htmlspecialchars ($text)."<br>\n");
	flush ();
}

function printErrors () {
	global $infoText, $errText, $successText;
	if (!empty ($infoText))
		printError ($infoText);
	if (!empty ($errText))
		printError ($errText);
	if (!empty ($successText))
		printError ($successText);			
}

/* Get parameter from session */
function getParamFromSession ($name, $default="") {
	$value=$default;
	if (!isParam ($name)) {		
		if (isset ($_SESSION[$name]))
			$value = $_SESSION[$name];
	}
	else
		$_SESSION[$name]=($value=getParam ($name));
	return $value;
}

/* Load properties from file */
function loadProperties () {
	global $CONF_VAR;
	$filename = getConf ("root_path")."/".
		getConf ("conf_path", "conf")."/".
		getConf ("property_file", "init.inc.php");
	if (file_exists ($filename)) {
		include ($filename);
	}
	return false;
}

/* Save properties into file */
function saveProperties () {
	global $CONF_VAR;
	$filename = 
		getConf ("root_path")."/".
		getConf ("conf_path", "conf")."/".
		getConf ("property_file", ".properties");

	$fh = fopen ($filename, "w+");
	if ($fh) {
		fwrite ($fh, "<?php\n// This file is generated automatically!\n// No not modify manually!\n\n");
		foreach ($_SESSION["DYNPAGE_CONF_VAR_ALL"] as $key => $value) {
			if ($key=="" || isset ($CONF_VAR[$key]))
				continue;
			fwrite ($fh, "\$_SESSION['DYNPAGE_CONF_VAR_ALL']['".$key."']=\"".$value."\";\n");
		}
		fwrite ($fh, "".chr(63).">\n");
		fclose ($fh);
		chmod  ($filename, 0666);
		return true;
	}
	return false;
}

/* Refresh configuration data */
function refreshConfiguration () {
	loadProperties ();
	$_SESSION["fCfgLoaded"] = 1;
}

/* Set new configuration var  */
function setConf ($key, $value) {	
	global $CONF_VAR;
	$_SESSION["DYNPAGE_CONF_VAR_ALL"][$key]=$value;
	if (!isset ($CONF_VAR["is_demo"]) || $CONF_VAR["is_demo"]==0)
		saveProperties ();
}

/* Get config variable from config_global.inc.php or db*/
function getConf ($key, $default="") {
	global $CONF_VAR;
	// Check file properties firstly
	if (isset ($CONF_VAR[$key])) 
		return $CONF_VAR[$key];
	return getArrayValue ($_SESSION, "DYNPAGE_CONF_VAR_ALL", $default, $key);	
}

/* Load configuration if needed */
if (!isset ($_SESSION["DYNPAGE_CONF_VAR_ALL"]))
	$_SESSION["DYNPAGE_CONF_VAR_ALL"]=$CONF_VAR;

if (!isset($_SESSION["fCfgLoaded"])) 
	refreshConfiguration ();

/* Set default variables / constants */
$cmd  = getParam ("cmd");
define ("ROOT_PATH", getConf  ("root_path"));
?>
